unit headersutils;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil;

var
  gJavaPackageName: string;
  gJavaProjectName: string;
  gAppName: string; // used in res/values/strings.xml for the icon text
  gHeadersDir: string;
  gJavaDestDir: string;

  gProjectDir: string;
  gAndroidSDKDir: string;

  gPATHJava: string;
  gApacheAnt:string;
  gSDKPlatform: string;

const
  //cAndroidPipesComm = 'AndroidPipesComm.java';
  cLCLActivity = 'LCLActivity.java';
  cAndroidlcltest = 'androidlcltest';

procedure CreateNewAndroidProject();
function Replace(S, Old, New: String): String;

implementation

function Replace(S, Old, New: String): String;
var i: integer;
begin
  Result := '';
  for i:=1 to Length(S) do begin
     if S[i]=Old then Result := Result + New
     else Result := Result + S[i];
  end;
end;

procedure AddJavaBuildFiles(); forward;
procedure AddResFiles(); forward;
procedure AddAndroidPipesComm(); forward;  //dodavanje java scripte
procedure AddBatchFile(); forward;

procedure CreateNewAndroidProject();
var
  Str: string;
begin
  // Creates the directory structure
  ForceDirectories(gProjectDir + 'android');
  ForceDirectories(gProjectDir + 'android' + PathDelim + 'libs' + PathDelim + 'armeabi');
  ForceDirectories(gProjectDir + 'android' + PathDelim + 'res' + PathDelim + 'values');
  CreateDir(gProjectDir + 'android' + PathDelim + 'res' + PathDelim + 'drawable-ldpi');

  Str := StringReplace(gJavaPackageName, '.', PathDelim, [rfReplaceAll]);
  ForceDirectories(gProjectDir + 'android' + PathDelim + 'src' + PathDelim + Str);

  // Now add the Java build files
  AddJavaBuildFiles();

  // Add the files in the res directory
  AddResFiles();

  //AddAndroidPipesComm  AddAndroidPipesComm.java
  AddAndroidPipesComm();

  //Add bath file compile.bat and send_emulator.bat
  AddBatchFile();
end;

procedure AddJavaBuildFiles();
var
  lFile: TStringList;
begin
  lFile := TStringList.Create;
  try
    lFile.Add('<?xml version="1.0" encoding="utf-8"?>');
    lFile.Add('<manifest xmlns:android="http://schemas.android.com/apk/res/android"');
    lFile.Add('  package="' + gJavaPackageName + '"');
    lFile.Add('  android:versionCode="1"');
    lFile.Add('  android:versionName="1.0">');
    lFile.Add('  <uses-permission android:name="android.permission.VIBRATE" />');
    lFile.Add('  <uses-permission android:name="android.permission.SEND_SMS" />');
    lFile.Add('  <uses-permission android:name="android.permission.ACCESS_FINE_LOCATION" />');
    lFile.Add('  <uses-permission android:name="android.permission.INTERNET" />');
    lFile.Add('  <uses-permission android:name="android.permission.WRITE_EXTERNAL_STORAGE" />');
    lFile.Add('  <supports-screens android:smallScreens="true"  ');
    lFile.Add('                    android:normalScreens="true" ');
    lFile.Add('                    android:largeScreens="true"  ');
    lFile.Add('                    android:anyDensity="true" /> ');
    lFile.Add('  <uses-sdk android:minSdkVersion="4" ');
    lFile.Add('            android:targetSdkVersion="4" />');
    lFile.Add('  <application android:label="@string/app_name" ');
    lFile.Add('               android:icon="@drawable/icon"');
    lFile.Add('    android:debuggable="True">');
   // lFile.Add('    <activity android:name="' + gJavaProjectName + '"');
    lFile.Add('    <activity android:name="LCLActivity"');
    lFile.Add('      android:configChanges="orientation|keyboardHidden"');
    lFile.Add('      android:label="@string/app_name"');
    lFile.Add('      android:launchMode="singleTop">');
    lFile.Add('      <intent-filter>');
    lFile.Add('        <action android:name="android.intent.action.MAIN" />');
    lFile.Add('        <category android:name="android.intent.category.LAUNCHER" />');
    lFile.Add('      </intent-filter>');
    lFile.Add('    </activity>');
    lFile.Add('  </application>');
    lFile.Add('</manifest>');
    lFile.SaveToFile(gProjectDir + 'android' + PathDelim + 'AndroidManifest.xml');

    lFile.Clear;
    lFile.Add('# This file is used to override default values used by the Ant build system.');
    lFile.Add('#');
    lFile.Add('# This file must be checked in Version Control Systems, as it is ');
    lFile.Add('# integral to the build system of your project.   ');
    lFile.Add('');
    lFile.Add('# This file is only used by the Ant script. ');
    lFile.Add('');
    lFile.Add('# You can use this to override default values such as        ');
    lFile.Add('#  ''source.dir'' for the location of your java source folder and          ');
    lFile.Add('#  ''out.dir'' for the location of your output folder.  ');
    lFile.Add('');
    lFile.Add('# You can also use it define how the release builds are signed by declaring    ');
    lFile.Add('# the following properties:   ');
    lFile.Add('#  ''key.store'' for the location of your keystore and        ');
    lFile.Add('#  ''key.alias'' for the name of the key to use.    ');
    lFile.Add('# The password will be asked during the build when you use the ''release'' target.       ');
    lFile.SaveToFile(gProjectDir + 'android' + PathDelim + 'build.properties');

    lFile.Clear;
    lFile.Add('<?xml version="1.0" encoding="UTF-8"?>');
    lFile.Add('<project name="'+gJavaProjectName+'" default="help">');
    lFile.Add('<property file="local.properties" />');
    lFile.Add('<property file="build.properties" />');
    lFile.Add('<property file="default.properties" />');
    lFile.Add('<path id="android.antlibs">');
    lFile.Add('<pathelement path="${sdk.dir}/tools/lib/anttasks.jar" />');
    lFile.Add('<pathelement path="${sdk.dir}/tools/lib/sdklib.jar" />');
    lFile.Add('<pathelement path="${sdk.dir}/tools/lib/androidprefs.jar" />');
    lFile.Add('</path>');
    lFile.Add('<import file="${sdk.dir}/tools/ant/build.xml" />');
    lFile.Add('');
    lFile.Add('</project>');
    lFile.SaveToFile(gProjectDir + 'android' + PathDelim + 'build.xml');

    lFile.Clear;
    lFile.Add('# Project target.');
    lFile.Add('target=android-8');
    lFile.SaveToFile(gProjectDir + 'android' + PathDelim + 'project.properties');
    lFile.SaveToFile(gProjectDir + 'android' + PathDelim + 'default.properties');
    lFile.Clear;
    {$ifdef linux}
    {$else}
     lFile.Add('sdk.dir=' + Replace(gAndroidSDKDir, '\', '\\'));
    {$endif}

    lFile.SaveToFile(gProjectDir + 'android' + PathDelim + 'local.properties');
  finally
    lFile.Free;
  end;
end;

procedure AddResFiles();
var
  lFile: TStringList;
begin
  lFile := TStringList.Create;
  try
    lFile.Add('<?xml version="1.0" encoding="utf-8"?>');
    lFile.Add('<resources>');
    lFile.Add('    <string name="app_name">' + gAppName + '</string>');
    lFile.Add('</resources>');
    lFile.SaveToFile(gProjectDir + 'android' + PathDelim + 'res' + PathDelim + 'values' + PathDelim + 'strings.xml');
  finally
    lFile.Free;
  end;

  // Now the default icon
  FileUtil.CopyFile(gHeadersDir + 'icon.png',
  gProjectDir + 'android' + PathDelim + 'res' + PathDelim + 'drawable-ldpi' + PathDelim + 'icon.png');
end;

procedure AddAndroidPipesComm();
var
  lFile: TStringList;
  target : string;
begin
  lFile := TStringList.Create;
  try
    //--------------- LCLActivity
    lFile.LoadFromFile(gHeadersDir + cLCLActivity);
    lFile.Insert(0, 'package ' + gJavaPackageName + ';');
    lFile.SaveToFile(gJavaDestDir +'\'+ cLCLActivity);

    //---------------- lpr
    lFile.Clear;

    lFile.Add('library '+gAppName+'; ');
    lFile.Add('');
    lFile.Add('{$mode objfpc}{$H+} ');
    lFile.Add('');
    lFile.Add('uses ');
    lFile.Add('  Interfaces, Forms, ');
    lFile.Add('  customdrawnint, customdrawn_android, customdrawndrawers; ');
    lFile.Add('');
    lFile.Add('exports ');
    target := Replace(gJavaPackageName, '.', '_');
    lFile.Add('  Java_com_pascal_lclproject_LCLActivity_LCLOnTouch name ''Java_'+target+'_LCLActivity_LCLOnTouch'', ');
    lFile.Add('  Java_com_pascal_lclproject_LCLActivity_LCLDrawToBitmap name ''Java_'+target+'_LCLActivity_LCLDrawToBitmap'', ');
    lFile.Add('  Java_com_pascal_lclproject_LCLActivity_LCLOnCreate name ''Java_'+target+'_LCLActivity_LCLOnCreate'', ');
    lFile.Add('  Java_com_pascal_lclproject_LCLActivity_LCLOnMessageBoxFinished name ''Java_'+target+'_LCLActivity_LCLOnMessageBoxFinished'', ');
    lFile.Add('  Java_com_pascal_lclproject_LCLActivity_LCLOnKey name ''Java_'+target+'_LCLActivity_LCLOnKey'', ');
    lFile.Add('  Java_com_pascal_lclproject_LCLActivity_LCLOnTimer name ''Java_'+target+'_LCLActivity_LCLOnTimer'', ');
    lFile.Add('  Java_com_pascal_lclproject_LCLActivity_LCLOnConfigurationChanged name ''Java_'+target+'_LCLActivity_LCLOnConfigurationChanged'', ');
    lFile.Add('  Java_com_pascal_lclproject_LCLActivity_LCLOnSensorChanged name ''Java_'+target+'_LCLActivity_LCLOnSensorChanged'', ');
    lFile.Add('  Java_com_pascal_lclproject_LCLActivity_LCLOnMenuAction name ''Java_'+target+'_LCLActivity_LCLOnMenuAction'', ');
    lFile.Add('  JNI_OnLoad name ''JNI_OnLoad'', ');
    lFile.Add('  JNI_OnUnload name ''JNI_OnUnload''; ');
    lFile.Add('');
    lFile.Add('procedure MyActivityOnCreate; ');
    lFile.Add('begin ');
    lFile.Add('  DefaultStyle := dsAndroid; ');
    lFile.Add('  Application.Initialize; ');
    lFile.Add(' // Application.CreateForm(TFMain, FMain); ');
    lFile.Add('  Application.Run; ');
    lFile.Add('end; ');
    lFile.Add('');
    lFile.Add('begin ');
    target := Replace(gJavaPackageName, '.', '/');
    lFile.Add('  CDWidgetset.ActivityClassName := '''+target+'/LCLActivity''; ');
    lFile.Add('  CDWidgetset.ActivityOnCreate := @MyActivityOnCreate; ');
    lFile.Add('end.');
    lFile.SaveToFile(gProjectDir +'\'+ gAppName + '.lpr');

    //----------------- lpi
    lFile.Clear;
    lFile.Add('<?xml version="1.0" encoding="UTF-8"?> ');
    lFile.Add('<CONFIG> ');
    lFile.Add('  <ProjectOptions> ');
    lFile.Add('    <Version Value="9"/> ');
    lFile.Add('    <PathDelim Value="\"/> ');
    lFile.Add('    <General> ');
    lFile.Add('      <Flags> ');
    lFile.Add('        <MainUnitHasCreateFormStatements Value="False"/> ');
    lFile.Add('        <MainUnitHasTitleStatement Value="False"/> ');
    lFile.Add('      </Flags> ');
    lFile.Add('      <SessionStorage Value="InProjectDir"/> ');
    lFile.Add('      <MainUnit Value="0"/> ');
    lFile.Add('      <Title Value="'+gAppName+'"/> ');
    lFile.Add('      <UseAppBundle Value="False"/> ');
    lFile.Add('      <ResourceType Value="res"/> ');
    lFile.Add('    </General> ');
    lFile.Add('    <i18n> ');
    lFile.Add('      <EnableI18N LFM="False"/> ');
    lFile.Add('    </i18n> ');
    lFile.Add('    <VersionInfo> ');
    lFile.Add('      <StringTable ProductVersion=""/> ');
    lFile.Add('    </VersionInfo> ');
    lFile.Add('    <MacroValues Count="1"> ');
    lFile.Add('      <Macro1 Name="LCLWidgetType" Value="customdrawn"/> ');
    lFile.Add('    </MacroValues> ');
    lFile.Add('    <BuildModes Count="1"> ');
    lFile.Add('      <Item1 Name="Default" Default="True"/> ');
    lFile.Add('      <SharedMatrixOptions Count="1"> ');
    lFile.Add('        <Item1 ID="174209357939" Modes="Default" Type="IDEMacro" MacroName="LCLWidgetType" Value="customdrawn"/> ');
    lFile.Add('      </SharedMatrixOptions> ');
    lFile.Add('    </BuildModes> ');
    lFile.Add('    <PublishOptions> ');
    lFile.Add('      <Version Value="2"/> ');
    lFile.Add('    </PublishOptions> ');
    lFile.Add('    <RunParams> ');
    lFile.Add('      <local> ');
    lFile.Add('        <FormatVersion Value="1"/> ');
    lFile.Add('      </local> ');
    lFile.Add('    </RunParams> ');
    lFile.Add('    <RequiredPackages Count="1"> ');
    lFile.Add('      <Item1> ');
    lFile.Add('        <PackageName Value="LCL"/> ');
    lFile.Add('      </Item1> ');
    lFile.Add('    </RequiredPackages> ');
    lFile.Add('    <Units Count="1"> ');
    lFile.Add('      <Unit0> ');
    lFile.Add('        <Filename Value="'+gAppName+'.lpr"/> ');
    lFile.Add('        <IsPartOfProject Value="True"/> ');
    lFile.Add('        <UnitName Value="'+gAppName+'"/> ');
    lFile.Add('      </Unit0> ');
    lFile.Add('    </Units> ');
    lFile.Add('  </ProjectOptions> ');
    lFile.Add('  <CompilerOptions> ');
    lFile.Add('    <Version Value="11"/> ');
    lFile.Add('    <PathDelim Value="\"/> ');
    lFile.Add('    <Target> ');
    lFile.Add('      <Filename Value="android\libs\armeabi\liblclapp.so"/> ');
    lFile.Add('    </Target> ');
    lFile.Add('    <SearchPaths> ');
    lFile.Add('      <IncludeFiles Value="$(ProjOutDir)"/> ');
    lFile.Add('      <Libraries Value="C:\codetyphon\binLibraries\android-4.0-api14-arm\"/> ');
    lFile.Add('      <UnitOutputDirectory Value="lib\$(TargetCPU)-$(TargetOS)"/> ');
    lFile.Add('    </SearchPaths> ');
    lFile.Add('    <CodeGeneration> ');
    lFile.Add('      <TargetCPU Value="arm"/> ');
    lFile.Add('      <TargetOS Value="android"/> ');
    lFile.Add('    </CodeGeneration> ');
    lFile.Add('    <Linking> ');
    lFile.Add('      <Debugging> ');
    lFile.Add('        <GenerateDebugInfo Value="False"/> ');
    lFile.Add('      </Debugging> ');
    lFile.Add('      <Options> ');
    lFile.Add('        <ExecutableType Value="Library"/> ');
    lFile.Add('      </Options> ');
    lFile.Add('    </Linking> ');
    lFile.Add('    <Other> ');
    lFile.Add('      <CompilerMessages> ');
    lFile.Add('        <MsgFileName Value=""/> ');
    lFile.Add('      </CompilerMessages> ');
    lFile.Add('      <CompilerPath Value="$(CompPath)"/> ');
    lFile.Add('    </Other> ');
    lFile.Add('  </CompilerOptions> ');
    lFile.Add('  <Debugging> ');
    lFile.Add('    <Exceptions Count="3"> ');
    lFile.Add('      <Item1> ');
    lFile.Add('        <Name Value="EAbort"/> ');
    lFile.Add('      </Item1> ');
    lFile.Add('      <Item2> ');
    lFile.Add('        <Name Value="ECodetoolError"/> ');
    lFile.Add('      </Item2> ');
    lFile.Add('      <Item3> ');
    lFile.Add('        <Name Value="EFOpenError"/> ');
    lFile.Add('      </Item3> ');
    lFile.Add('    </Exceptions> ');
    lFile.Add('  </Debugging> ');
    lFile.Add('</CONFIG> ');
    lFile.SaveToFile(gProjectDir +'\'+ gAppName + '.lpi');
  finally
    lFile.Free;
  end;
end;

procedure AddBatchFile();
var
  lFile: TStringList;
begin
  lFile := TStringList.Create;
  try
    lFile.Add('SET PATH='+gPATHJava+';'+gApacheAnt);
    lFile.Add('SET APK_SDK_PLATFORM='+gSDKPlatform);
    lFile.Add('');
    lFile.Add('rmdir bin /s /q');
    lFile.Add('rmdir gen /s /q');
    lFile.Add('');
    lFile.Add('ant debug');
    lFile.SaveToFile(gProjectDir + 'android' + PathDelim + 'build_debug_apk.bat');

    lFile.Clear;
    lFile.Add('SET PATH='+gAndroidSDKDir+'tools;'+gAndroidSDKDir+ 'platform-tools'+ PathDelim +';'+gPATHJava);
    lFile.Add('');
    //lFile.Add('keytool -genkeypair -dname "cn=Zeljko Cvijanovic, ou=ZELJUSszd, o=Sun, c=RS" -alias business -keypass 111111 -keystore LCLDebugKey.keystore -storepass 111111 -validity 180');
    //lFile.Add('move LCLDebugKey.keystore bin\LCLDebugKey.keystore');
    //lFile.Add('');
    //lFile.Add('');
    lFile.Add('adb uninstall '+gJavaPackageName);
    lFile.Add('adb -s emulator-5554 install bin'+ PathDelim +gJavaProjectName+'-debug.apk');
    lFile.SaveToFile(gProjectDir + 'android' + PathDelim + 'send_emulator.bat');

    lFile.Clear;
    lFile.Add('SET PATH='+gAndroidSDKDir+'tools;'+gAndroidSDKDir+ 'platform-tools'+ PathDelim +';'+gPATHJava);
    lFile.Add('');
    //lFile.Add('keytool -genkeypair -dname "cn=Zeljko Cvijanovic, ou=ZELJUSszd, o=Sun, c=RS" -alias business -keypass 111111 -keystore LCLDebugKey.keystore -storepass 111111 -validity 180');
    //lFile.Add('move LCLDebugKey.keystore bin\LCLDebugKey.keystore');
    //lFile.Add('');
    //lFile.Add('');
    lFile.Add('adb uninstall '+gJavaPackageName);
    lFile.Add('adb install bin'+ PathDelim +gJavaProjectName+'-debug.apk');
    lFile.SaveToFile(gProjectDir + 'android' + PathDelim + 'send_tablet.bat');

  finally
    lFile.Free;
  end;
end;

end.

